from web3 import Web3
from helios_web3.hls import Hls
from web3._utils.empty import empty

class HeliosWeb3(Web3):
    def __init__(self, provider=empty, middlewares=None, modules=None, ens=empty):
        if modules is None:
            modules = {'hls': (Hls,)}
        else:
            modules['hls'] = (Hls,)

        super().__init__(provider=provider, middlewares=middlewares, modules=modules, ens=ens)



import time

from eth_account import Account as EthAccount
from eth_utils.curried import (
    combomethod,
    keccak,
)

from collections import (
    Mapping,
)

from cytoolz import (
    dissoc,
)
from eth_keys.datatypes import PrivateKey

from typing import (
    Tuple,
    List,
)

from hvm.vm.forks.helios_testnet.transactions import HeliosTestnetTransaction
# fields = [
#         ('nonce', big_endian_int),
#         ('gas_price', big_endian_int),
#         ('gas', big_endian_int),
#         ('to', address),
#         ('value', big_endian_int),
#         ('data', binary),
#         ('v', big_endian_int),
#         ('r', big_endian_int),
#         ('s', big_endian_int),
#     ]

from hvm.vm.forks.helios_testnet.transactions import HeliosTestnetReceiveTransaction
# fields = [
#         ('sender_block_hash', hash32),
#         ('send_transaction_hash', hash32),
#         ('is_refund', boolean),
#         ('remaining_refund', big_endian_int)
#     ]

from hvm.vm.forks.helios_testnet.blocks import HeliosTestnetBlock

from hvm.db.trie import make_trie_root_and_nodes

from hvm.rlp.headers import (
    MicroBlockHeader,
    BlockHeader)

# fields = [
#     ('chain_address', address),
#     ('parent_hash', hash32),
#     ('transaction_root', trie_root),
#     ('receive_transaction_root', trie_root),
#     ('block_number', big_endian_int),
#     ('timestamp', big_endian_int),
#     ('extra_data', binary),
#     ('reward_hash', hash32),
#     ('v', big_endian_int),
#     ('r', big_endian_int),
#     ('s', big_endian_int),
# ]



class Account(EthAccount):

    @combomethod
    def signBlock(self, send_transaction_dicts: List[dict], receive_transaction_dicts: List[dict], header_dict: dict, private_key) -> Tuple[bytes]:
        '''

        transaction = {
                    # Note that the address must be in checksum format:
                    'to': '0xF0109fC8DF283027b6285cc889F5aA624EaC1F55',
                    'value': 1000000000,
                    'gas': 2000000,
                    'gasPrice': 234567897654321,
                    'nonce': 0,
                    'chainId': 1
                }

         receive_transaction = {
                'senderBlockHash',
                'sendTransactionHash',
                'isRefund',
                'remainingRefund'
            }

        header = {
                'parentHash',
                'blockNumber',
                'extraData',
            }

        :param send_transaction_dicts:
        :param receive_transaction_dicts:
        :param reward_bundle:
        :param private_key:
        :return:
        '''

        account = self.privateKeyToAccount(private_key)
        chain_id = 1
        send_transactions = []
        for transaction_dict in send_transaction_dicts:
            tx = HeliosTestnetTransaction(nonce = transaction_dict.nonce,
                                          gas_price = transaction_dict.gasPrice,
                                          gas = transaction_dict.gas,
                                          to = transaction_dict.to,
                                          value = transaction_dict.value,
                                          )
            chain_id = transaction_dict.chainId
            signed_tx = tx.get_signed(account._key_obj, chain_id)
            send_transactions.append(signed_tx)

        receive_transactions = []
        for receive_transaction_dict in receive_transaction_dicts:
            tx = HeliosTestnetReceiveTransaction(sender_block_hash = receive_transaction_dict.senderBlockHash,
                                                 send_transaction_hash = receive_transaction_dict.sendTransactionHash,
                                                 is_refund = receive_transaction_dict.isRefund,
                                                 remaining_refund = receive_transaction_dict.remainingRefund)
            receive_transactions.append(tx)


        send_tx_root_hash, _ = make_trie_root_and_nodes(send_transactions)
        receive_tx_root_hash, _ = make_trie_root_and_nodes(receive_transactions)

        chain_address = account.address

        timestamp = int(time.time())

        header = BlockHeader(chain_address = chain_address,
                              parent_hash = header_dict.parentHash,
                              transaction_root = send_tx_root_hash,
                              receive_transaction_root = receive_tx_root_hash,
                              block_number = header_dict.blockNumber,
                              timestamp = timestamp,
                              extra_data = header_dict.extraData)

        signed_header = header.get_signed(account._key_obj, chain_id)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import (
    find_packages,
    setup,
)


setup(
    name='helios_web3',
    version='5.0.4',
    description="""Helios version of Web3.py""",
    long_description_markdown_filename='README.rst',
    author='Tommy Mckinnon',
    author_email='tommy@heliosprotocol.io',
    url='https://github.com/Helios-Protocol/helios_web3.py',
    include_package_data=True,
    install_requires=[
        "web3>=5.0.0a9,<6.0.0",
        "eth-abi>=2.0.0b6,<3.0.0",
        "eth-account>=0.2.1,<0.4.0",
        "eth-hash[pycryptodome]>=0.2.0,<1.0.0",
        "eth-typing>=2.0.0,<3.0.0",
        "eth-utils>=1.3.0,<2.0.0",
        "ethpm>=0.1.4a12,<1.0.0",
        "hexbytes>=0.1.0,<1.0.0",
        "lru-dict>=1.1.6,<2.0.0",
        "requests>=2.16.0,<3.0.0",
        "pypiwin32>=223;platform_system=='Windows'",
        "py-helios-node>=0.2.0-alpha.33"
    ],
    setup_requires=['setuptools-markdown'],
    python_requires='>=3.6,<4',
    license="MIT",
    zip_safe=False,
    keywords='ethereum, helios protocol',
    packages=find_packages(exclude=["tests", "tests.*"]),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)


import sys
import warnings

if (3, 5) <= sys.version_info < (3, 6):
    warnings.warn(
        "Support for Python 3.5 will be removed in web3.py v5",
        category=DeprecationWarning,
        stacklevel=2)

if sys.version_info < (3, 5):
    raise EnvironmentError(
        "Python 3.5 or above is required. "
        "Note that support for Python 3.5 will be removed in web3.py v5")

from helios_web3.main import HeliosWeb3